@@ -10,6 +10,7 @@ ONNX (native) format
 import os
 import yaml
 import numpy as np
+from pathlib import Path
 
 import pandas as pd
 
@@ -20,7 +21,6 @@ import mlflow.tracking
 from mlflow.exceptions import MlflowException
 from mlflow.models.signature import ModelSignature
 from mlflow.models.utils import ModelInputExample, _save_example
-from mlflow.protos.databricks_pb2 import RESOURCE_ALREADY_EXISTS
 from mlflow.tracking.artifact_utils import _download_artifact_from_uri
 from mlflow.utils.annotations import experimental
 from mlflow.utils.environment import (
@@ -35,10 +35,16 @@ from mlflow.utils.environment import (
 from mlflow.utils.requirements_utils import _get_pinned_requirement
 from mlflow.utils.file_utils import write_to
 from mlflow.utils.docstring_utils import format_docstring, LOG_MODEL_PARAM_DOCS
-from mlflow.utils.model_utils import _get_flavor_configuration
+from mlflow.utils.model_utils import (
+    _get_flavor_configuration,
+    _validate_and_copy_code_paths,
+    _add_code_from_conf_to_system_path,
+    _validate_and_prepare_target_save_path,
+)
 from mlflow.tracking._model_registry import DEFAULT_AWAIT_MAX_SLEEP_SECONDS
 
 FLAVOR_NAME = "onnx"
+ONNX_EXECUTION_PROVIDERS = ["CUDAExecutionProvider", "CPUExecutionProvider"]
 
 
 def get_default_pip_requirements():
@@ -76,11 +82,13 @@ def save_model(
     onnx_model,
     path,
     conda_env=None,
+    code_paths=None,
     mlflow_model=None,
     signature: ModelSignature = None,
     input_example: ModelInputExample = None,
     pip_requirements=None,
     extra_pip_requirements=None,
+    onnx_execution_providers=None,
 ):
     """
     Save an ONNX model to a path on the local file system.
@@ -88,6 +96,9 @@ def save_model(
     :param onnx_model: ONNX model to be saved.
     :param path: Local path where the model is to be saved.
     :param conda_env: {{ conda_env }}
+    :param code_paths: A list of local filesystem paths to Python file dependencies (or directories
+                       containing file dependencies). These files are *prepended* to the system
+                       path when the model is loaded.
     :param mlflow_model: :py:mod:`mlflow.models.Model` this flavor is being added to.
 
     :param signature: :py:class:`ModelSignature <mlflow.models.ModelSignature>`
@@ -111,17 +122,24 @@ def save_model(
                           by converting it to a list. Bytes are base64-encoded.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param onnx_execution_providers: List of strings defining onnxruntime execution providers.
+                                     Defaults to example:
+                                     ``['CUDAExecutionProvider', 'CPUExecutionProvider']``
+                                     This uses GPU preferentially over CPU.
+                                     See onnxruntime API for further descriptions:
+                                     https://onnxruntime.ai/docs/execution-providers/
     """
     import onnx
 
+    if onnx_execution_providers is None:
+        onnx_execution_providers = ONNX_EXECUTION_PROVIDERS
+
     _validate_env_arguments(conda_env, pip_requirements, extra_pip_requirements)
 
     path = os.path.abspath(path)
-    if os.path.exists(path):
-        raise MlflowException(
-            message="Path '{}' already exists".format(path), error_code=RESOURCE_ALREADY_EXISTS
-        )
-    os.makedirs(path)
+    _validate_and_prepare_target_save_path(path)
+    code_dir_subpath = _validate_and_copy_code_paths(code_paths, path)
+
     if mlflow_model is None:
         mlflow_model = Model()
     if signature is not None:
@@ -135,9 +153,19 @@ def save_model(
     onnx.save_model(onnx_model, model_data_path)
 
     pyfunc.add_to_model(
-        mlflow_model, loader_module="mlflow.onnx", data=model_data_subpath, env=_CONDA_ENV_FILE_NAME
+        mlflow_model,
+        loader_module="mlflow.onnx",
+        data=model_data_subpath,
+        env=_CONDA_ENV_FILE_NAME,
+        code=code_dir_subpath,
+    )
+    mlflow_model.add_flavor(
+        FLAVOR_NAME,
+        onnx_version=onnx.__version__,
+        data=model_data_subpath,
+        providers=onnx_execution_providers,
+        code=code_dir_subpath,
     )
-    mlflow_model.add_flavor(FLAVOR_NAME, onnx_version=onnx.__version__, data=model_data_subpath)
     mlflow_model.save(os.path.join(path, MLMODEL_FILE_NAME))
 
     if conda_env is None:
@@ -182,10 +210,48 @@ def _load_model(model_file):
 
 
 class _OnnxModelWrapper:
-    def __init__(self, path):
+    def __init__(self, path, providers=None):
         import onnxruntime
 
-        self.rt = onnxruntime.InferenceSession(path)
+        # Get the model meta data from the MLModel yaml file which may contain the providers
+        # specification.
+        local_path = str(Path(path).parent)
+        model_meta = Model.load(os.path.join(local_path, MLMODEL_FILE_NAME))
+
+        # Check if the MLModel config has the providers meta data
+        if "providers" in model_meta.flavors.get(FLAVOR_NAME).keys():
+            providers = model_meta.flavors.get(FLAVOR_NAME)["providers"]
+        # If not, then default to the predefined list.
+        else:
+            providers = ONNX_EXECUTION_PROVIDERS
+
+        # NOTE: Some distributions of onnxruntime require the specification of the providers
+        # argument on calling. E.g. onnxruntime-gpu. The package import call does not differnetiate
+        #  which architecture specific version has been installed, as all are imported with
+        # onnxruntime. onnxruntime documentation says that from v1.9.0 some distributions require
+        #  the providers list to be provided on calling an InferenceSession. Therefore the try
+        #  catch structure below attempts to create an inference session with just the model path
+        #  as pre v1.9.0. If that fails, it will use the providers list call.
+        # At the moment this is just CUDA and CPU, and probably should be expanded.
+        # A method of user customisation has been provided by adding a variable in the save_model()
+        # function, which allows the ability to pass the list of execution providers via a
+        # optional argument e.g.
+        #
+        # mlflow.onnx.save_model(..., providers=['CUDAExecutionProvider'...])
+        #
+        # For details of the execution providers construct of onnxruntime, see:
+        # https://onnxruntime.ai/docs/execution-providers/
+        #
+        # For a information on how execution providers are used with onnxruntime InferenceSession,
+        # see the API page below:
+        # https://onnxruntime.ai/docs/api/python/api_summary.html#id8
+        #
+
+        try:
+            self.rt = onnxruntime.InferenceSession(path)
+        except ValueError:
+            self.rt = onnxruntime.InferenceSession(path, providers=providers)
+
         assert len(self.rt.get_inputs()) >= 1
         self.inputs = [(inp.name, inp.type) for inp in self.rt.get_inputs()]
         self.output_names = [outp.name for outp in self.rt.get_outputs()]
@@ -280,7 +346,7 @@ class _OnnxModelWrapper:
 
 def _load_pyfunc(path):
     """
-    Load PyFunc implementation. Called by ``pyfunc.load_pyfunc``.
+    Load PyFunc implementation. Called by ``pyfunc.load_model``.
     """
     return _OnnxModelWrapper(path)
 
@@ -311,6 +377,7 @@ def load_model(model_uri, dst_path=None):
     """
     local_model_path = _download_artifact_from_uri(artifact_uri=model_uri, output_path=dst_path)
     flavor_conf = _get_flavor_configuration(model_path=local_model_path, flavor_name=FLAVOR_NAME)
+    _add_code_from_conf_to_system_path(local_model_path, flavor_conf)
     onnx_model_artifacts_path = os.path.join(local_model_path, flavor_conf["data"])
     return _load_model(model_file=onnx_model_artifacts_path)
 
@@ -321,12 +388,14 @@ def log_model(
     onnx_model,
     artifact_path,
     conda_env=None,
+    code_paths=None,
     registered_model_name=None,
     signature: ModelSignature = None,
     input_example: ModelInputExample = None,
     await_registration_for=DEFAULT_AWAIT_MAX_SLEEP_SECONDS,
     pip_requirements=None,
     extra_pip_requirements=None,
+    onnx_execution_providers=None,
 ):
     """
     Log an ONNX model as an MLflow artifact for the current run.
@@ -334,6 +403,9 @@ def log_model(
     :param onnx_model: ONNX model to be saved.
     :param artifact_path: Run-relative artifact path.
     :param conda_env: {{ conda_env }}
+    :param code_paths: A list of local filesystem paths to Python file dependencies (or directories
+                       containing file dependencies). These files are *prepended* to the system
+                       path when the model is loaded.
     :param registered_model_name: If given, create a model version under
                                   ``registered_model_name``, also creating a registered model if one
                                   with the given name does not exist.
@@ -362,6 +434,12 @@ def log_model(
                             waits for five minutes. Specify 0 or None to skip waiting.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param onnx_execution_providers: List of strings defining onnxruntime execution providers.
+                                     Defaults to example:
+                                     ['CUDAExecutionProvider', 'CPUExecutionProvider']
+                                     This uses GPU preferentially over CPU.
+                                     See onnxruntime API for further descriptions:
+                                     https://onnxruntime.ai/docs/execution-providers/
     :return: A :py:class:`ModelInfo <mlflow.models.model.ModelInfo>` instance that contains the
              metadata of the logged model.
     """
@@ -370,10 +448,12 @@ def log_model(
         flavor=mlflow.onnx,
         onnx_model=onnx_model,
         conda_env=conda_env,
+        code_paths=code_paths,
         registered_model_name=registered_model_name,
         signature=signature,
         input_example=input_example,
         await_registration_for=await_registration_for,
         pip_requirements=pip_requirements,
         extra_pip_requirements=extra_pip_requirements,
+        onnx_execution_providers=onnx_execution_providers,
     )
